********************************************************************************
*       Do file for Analysis Tanzania Consumption Experiment 2021/2022         *
*			  Program to Automate Final Report Table Creation		           *
********************************************************************************

clear all

set linesize 90
set maxvar 8000
set matsize 3500

capture program drop starmaker  // define the starmaker program  
program define starmaker 		//starmaker was created by Greg Haugan at NORC
				local coef: display %9.`2'f round(_b[`1'], 0.001) // Store regression coefficient
				local se: display %9.2f round(_se[`1'], 0.001) // Store standard error
				local t = abs(_b[`1']/_se[`1']) // Compute t-statistic
				local p = 2*ttail(e(df_r), `t') // Compute p-value
						if `p' > 0.10{ // Significance stars
							local stars = "n/s" //no stars if p is above 0.05
						}
						if `p' <= 0.10{
							local stars = "*"
						}
						if `p' <= 0.05{
							local stars = "**"
						}				
						if `p' <= 0.01{
							local stars = "***"
						}
						
global coef = ltrim("`coef'")
global se = ltrim("`se'")
global stars = ltrim("`stars'")
global pval : di %9.3f `p'
end

********************************************************************************
** Table 1. Basic demographic characteristics 
********************************************************************************

capture program drop ftftable 	// define the ftftable program
program define ftftable 		// ftftable was created by Greg Haugan at NORC 

preserve
local row = 3
foreach outcome of varlist $indicators{

	*** Results
svy: mean `outcome' if treatment == 0
matrix b = e(b)
local mean : di %9.1f b[1,1]
putexcel B`row' = "`mean'" E`row' = "`mean'" 

svy: mean `outcome' if treatment == 1
matrix b = e(b)
local mean : di %9.1f b[1,1]
putexcel C`row' = "`mean'" H`row' = "`mean'" 

svy: mean `outcome' if treatment == 2
matrix b = e(b)
local mean : di %9.1f b[1,1]
putexcel F`row' = "`mean'" I`row' = "`mean'" 

	***	Significance tests 	
svy: reg `outcome' treatment if inlist(treatment,0,1)
starmaker treatment 3
putexcel D`row' = "$stars"

svy: reg `outcome' treatment if inlist(treatment,0,2)
starmaker treatment 3
putexcel G`row' = "$stars"

svy: reg `outcome' treatment if inlist(treatment,1,2)
starmaker treatment 3
putexcel J`row' = "$stars"

local row = `row' + 1

}
restore
end   

*---------------------------------------------*
*          SETUP: DATA AND VARIABLES          *			
*---------------------------------------------*

use "$data\cons_analysis.dta", clear
putexcel set "$tables\Tables and Figures - Consumption Experiment.xlsx", sheet(T1) modify

svyset [pw=hhweight], psu(cluster) strata(strata) singleunit(centered)
global indicators hhsize adulteq num_adult18 num_male num_female
ftftable "Tables and Figures - Consumption Experiment.xlsx" 2 

******************************************************************************************
** Table 1. Basic demographic characteristics - Head of household (age and gender)
******************************************************************************************
		        
capture program drop ftftable 	// define the ftftable program
program define ftftable 		// ftftable was created by Greg Haugan at NORC 

preserve
local row = 9
foreach outcome of varlist $indicators{
	
	*** Results 
svy: mean `outcome' if treatment == 0
matrix b = e(b)
local mean : di %9.1f b[1,1]
putexcel B`row' = "`mean'" E`row' = "`mean'" 

svy: mean `outcome' if treatment == 1
matrix b = e(b)
local mean : di %9.1f b[1,1]
putexcel C`row' = "`mean'" H`row' = "`mean'" 

svy: mean `outcome' if treatment == 2
matrix b = e(b)
local mean : di %9.1f b[1,1]
putexcel F`row' = "`mean'" I`row' = "`mean'" 

	***	Significance tests	
svy: reg `outcome' treatment if inlist(treatment,0,1)
starmaker treatment 3
putexcel D`row' = "$stars"

svy: reg `outcome' treatment if inlist(treatment,0,2)
starmaker treatment 3
putexcel G`row' = "$stars"

svy: reg `outcome' treatment if inlist(treatment,1,2)
starmaker treatment 3
putexcel J`row' = "$stars"

local row = `row' + 1

}
restore
end 

*---------------------------------------------*
*          SETUP: DATA AND VARIABLES          *			
*---------------------------------------------*

use "$data\cons_analysis.dta", clear
// Table prep:
	recode femalehead (1=100)
	recode seceduc_head (1=100)
	recode emp_head (1=100)

putexcel set "$tables\Tables and Figures - Consumption Experiment.xlsx", sheet(T1) modify

svyset [pw=hhweight], psu(cluster) strata(strata) singleunit(centered)
global indicators age_head femalehead seceduc_head emp_head wage_head nfe_head
ftftable "Tables and Figures - Consumption Experiment.xlsx" 2 

******************************************************************************************
** Table 1. Basic demographic characteristics - Main respondent (age and gender)
******************************************************************************************
  
capture program drop ftftable 	// define the ftftable program
program define ftftable 		// ftftable was created by Greg Haugan at NORC 

preserve
local row = 16
foreach outcome of varlist $indicators{

	*** Results
svy: mean `outcome' if treatment == 0
matrix b = e(b)
local mean : di %9.1f b[1,1]
putexcel B`row' = "`mean'" E`row' = "`mean'" 

svy: mean `outcome' if treatment == 1
matrix b = e(b)
local mean : di %9.1f b[1,1]
putexcel C`row' = "`mean'" H`row' = "`mean'" 

svy: mean `outcome' if treatment == 2
matrix b = e(b)
local mean : di %9.1f b[1,1]
putexcel F`row' = "`mean'" I`row' = "`mean'" 

	***	Significance tests	
svy: reg `outcome' treatment if inlist(treatment,0,1)
starmaker treatment 3
putexcel D`row' = "$stars"

svy: reg `outcome' treatment if inlist(treatment,0,2)
starmaker treatment 3
putexcel G`row' = "$stars"

svy: reg `outcome' treatment if inlist(treatment,1,2)
starmaker treatment 3
putexcel J`row' = "$stars"

local row = `row' + 1

}
restore
end   

*---------------------------------------------*
*          SETUP: DATA AND VARIABLES          *			
*---------------------------------------------*

use "$data\cons_analysis.dta", clear
putexcel set "$tables\Tables and Figures - Consumption Experiment.xlsx", sheet(T1) modify

svyset [pw=hhweight], psu(cluster) strata(strata) singleunit(centered)
global indicators age_resp sex_resp seceduc_resp emp_resp wage_resp nfe_resp
ftftable "Tables and Figures - Consumption Experiment.xlsx" 2 

******************************************************************************************
** Table 1. Basic demographic characteristics - Number of households
******************************************************************************************f

capture program drop ftftable 	// define the ftftable program
program define ftftable 		// ftftable was created by Greg Haugan at NORC 

preserve
local row = 23
foreach outcome of varlist $indicators{

	*** Results
svy: mean `outcome' if treatment == 0
matrix b = e(b)
matrix N = e(N)
local N : di %9.0f N[1,1]
putexcel B`row' = "`N'" E`row' = "`N'" 

svy: mean `outcome' if treatment == 1
matrix b = e(b)
matrix N = e(N)
local N : di %9.0f N[1,1]
putexcel C`row' = "`N'" H`row' = "`N'" 

svy: mean `outcome' if treatment == 2
matrix b = e(b)
matrix N = e(N)
local N : di %9.0f N[1,1]
putexcel F`row' = "`N'" I`row' = "`N'"

local row = `row' + 1

}
restore
end   

*---------------------------------------------*
*          SETUP: DATA AND VARIABLES          *			
*---------------------------------------------*

use "$data\cons_analysis.dta", clear
putexcel set "$tables\Tables and Figures - Consumption Experiment.xlsx", sheet(T1) modify

svyset [pw=hhweight], psu(cluster) strata(strata) singleunit(centered)
global indicators hhsize 
ftftable "Tables and Figures - Consumption Experiment.xlsx" 2  

***************************************************************************************************
** Table 2. Basic housing characteristics, by treatment
***************************************************************************************************

capture program drop ftftable 	// define the ftftable program
program define ftftable 		// ftftable was created by Greg Haugan at NORC 

preserve
local row = 3
foreach outcome of varlist $indicators{

	*** Results
svy: mean `outcome' if treatment == 0
matrix b = e(b)
local mean : di %9.1f b[1,1]
putexcel B`row' = "`mean'" E`row' = "`mean'" 

svy: mean `outcome' if treatment == 1
matrix b = e(b)
local mean : di %9.1f b[1,1]
putexcel C`row' = "`mean'" H`row' = "`mean'" 

svy: mean `outcome' if treatment == 2
matrix b = e(b)
local mean : di %9.1f b[1,1]
putexcel F`row' = "`mean'" I`row' = "`mean'" 

	***	Significance tests 	
svy: reg `outcome' treatment if inlist(treatment,0,1)
starmaker treatment 3
putexcel D`row' = "$stars"

svy: reg `outcome' treatment if inlist(treatment,0,2)
starmaker treatment 3
putexcel G`row' = "$stars"

svy: reg `outcome' treatment if inlist(treatment,1,2)
starmaker treatment 3
putexcel J`row' = "$stars"

local row = `row' + 1

}
restore
end   

*---------------------------------------------*
*          SETUP: DATA AND VARIABLES          *			
*---------------------------------------------*

use "$data\cons_analysis.dta", clear
// Table prep:
	tab dw_floor, gen(dw_floor_)
		recode dw_floor_* (1=100)
	tab dw_wall, gen(dw_wall_)
		recode dw_wall_* (1=100)
	tab dw_roof, gen(dw_roof_)
		recode dw_roof_* (1=100)
  
putexcel set "$tables\Tables and Figures - Consumption Experiment.xlsx", sheet(T2) modify

svyset [pw=hhweight], psu(cluster) strata(strata) singleunit(centered)
global indicators dw_floor_2 dw_wall_2 dw_roof_2 improved_rainy2 improved_dry safewater_rainy2 safewater_dry
ftftable "Tables and Figures - Consumption Experiment.xlsx" 2 

***************************************************************************************************
** Table 2. Basic housing characteristics, by treatment (drinking water, sanitation)
*************************************************************************************************** 
		 
capture program drop ftftable 	// define the ftftable program
program define ftftable 		// ftftable was created by Greg Haugan at NORC 

preserve
local row = 11
foreach outcome of varlist $indicators{

	*** Results
svy: mean `outcome' if treatment == 0
matrix b = e(b)
local mean : di %9.1f b[1,1]
putexcel B`row' = "`mean'" E`row' = "`mean'" 

svy: mean `outcome' if treatment == 1
matrix b = e(b)
local mean : di %9.1f b[1,1]
putexcel C`row' = "`mean'" H`row' = "`mean'" 

svy: mean `outcome' if treatment == 2
matrix b = e(b)
local mean : di %9.1f b[1,1]
putexcel F`row' = "`mean'" I`row' = "`mean'" 

	***	Significance tests 	
svy: reg `outcome' treatment if inlist(treatment,0,1)
starmaker treatment 3
putexcel D`row' = "$stars"

svy: reg `outcome' treatment if inlist(treatment,0,2)
starmaker treatment 3
putexcel G`row' = "$stars"

svy: reg `outcome' treatment if inlist(treatment,1,2)
starmaker treatment 3
putexcel J`row' = "$stars"

local row = `row' + 1

}
restore
end   

*---------------------------------------------*
*          SETUP: DATA AND VARIABLES          *			
*---------------------------------------------*

use "$data\cons_analysis.dta", clear
// Prep for table
	tab dwater_rainy, gen (dwater_rainy_)
	recode dwater_rainy_* (1=100)

putexcel set "$tables\Tables and Figures - Consumption Experiment.xlsx", sheet(T2) modify

svyset [pw=hhweight], psu(cluster) strata(strata) singleunit(centered)
global indicators dwater_rainy_* basicsanit num_rooms
ftftable "Tables and Figures - Consumption Experiment.xlsx" 2 

***************************************************************************************************
** Table 2. Basic housing characteristics, by treatment (number of households)
***************************************************************************************************

capture program drop ftftable 	// define the ftftable program
program define ftftable 		// ftftable was created by Greg Haugan at NORC 

preserve
local row = 25
foreach outcome of varlist $indicators{

	*** Results
svy: mean `outcome' if treatment == 0
matrix b = e(b)
matrix N = e(N)
local N : di %9.0f N[1,1]
putexcel B`row' = "`N'" E`row' = "`N'" 

svy: mean `outcome' if treatment == 1
matrix b = e(b)
matrix N = e(N)
local N : di %9.0f N[1,1]
putexcel C`row' = "`N'" H`row' = "`N'" 

svy: mean `outcome' if treatment == 2
matrix b = e(b)
matrix N = e(N)
local N : di %9.0f N[1,1]
putexcel F`row' = "`N'" I`row' = "`N'"
 
local row = `row' + 1

}
restore
end   

*---------------------------------------------*
*          SETUP: DATA AND VARIABLES          *			
*---------------------------------------------*

use "$data\cons_analysis.dta", clear
putexcel set "$tables\Tables and Figures - Consumption Experiment.xlsx", sheet(T2) modify

svyset [pw=hhweight], psu(cluster) strata(strata) singleunit(centered)
global indicators hhsize 
ftftable "Tables and Figures - Consumption Experiment.xlsx" 2 
  
********************************************************************************
** Table 3. Average number of food items consumed per household, by treatment
*******************************************************************************
          
capture program drop ftftable 	// define the ftftable program
program define ftftable 		// ftftable was created by Greg Haugan at NORC 

preserve
local row = 3
foreach outcome of varlist $indicators{

	*** Results
svy: mean `outcome' if treatment == 0
matrix b = e(b)
local mean : di %9.0f b[1,1]
putexcel B`row' = "`mean'" E`row' = "`mean'" 

svy: mean `outcome' if treatment == 1
matrix b = e(b)
local mean : di %9.0f b[1,1]
putexcel C`row' = "`mean'" H`row' = "`mean'" 

svy: mean `outcome' if treatment == 2
matrix b = e(b)
local mean : di %9.0f b[1,1]
putexcel F`row' = "`mean'" I`row' = "`mean'" 

	***	Significance tests	
svy: reg `outcome' treatment if inlist(treatment,0,1)
starmaker treatment 3
putexcel D`row' = "$stars"

svy: reg `outcome' treatment if inlist(treatment,0,2)
starmaker treatment 3
putexcel G`row' = "$stars"

svy: reg `outcome' treatment if inlist(treatment,1,2)
starmaker treatment 3
putexcel J`row' = "$stars"

local row = `row' + 1

}
restore
end   

*---------------------------------------------*
*          SETUP: DATA AND VARIABLES          *			
*---------------------------------------------*

use "$data\food_cat_012.dta", clear
	mmerge hhid using "$data\cons_analysis.dta", unmatched (master) ukeep(cluster strata hhweight)
	rename treat treatment
	drop _merge

putexcel set "$tables\Tables and Figures - Consumption Experiment.xlsx", sheet(T3) modify
svyset [pw=hhweight], psu(cluster) strata(strata) singleunit(centered)
global indicators items_num
ftftable "Tables and Figures - Consumption Experiment.xlsx" 2  

**********************************************************
* Table 4. Most common food items consumed, by treatment
**********************************************************

use "$data\cons_analysis_item.dta", clear
clear matrix

putexcel set "$tables\Tables and Figures - Consumption Experiment.xlsx", sheet(T4) modify

tab item if treatment==0, sort matcell(b)
mat b2=b[1..19,1]
mat B5 = matrix(b2/r(N))*100 
putexcel B5=matrix(B5)

tab item if treatment==1, sort matcell(b)
mat b2=b[1..12,1]
mat E5 = matrix(b2/r(N))*100 
putexcel E5=matrix(E5)

tab item if treatment==2, sort matcell(b)
mat b2=b[1..19,1]
mat H5 = matrix(b2/r(N))*100 
putexcel H5=matrix(H5)

********************************************************************************
** Table 5: Average monthly expenditure of food groups, by treatment (T0)
********************************************************************************  

capture program drop ftftable // define the ftftable program
program define ftftable //ftftable was created by Greg Haugan at NORC 

preserve
putexcel set "$tables\Tables and Figures - Consumption Experiment.xlsx", sheet(T5) modify

local row = 3
foreach outcome of varlist $indicators {

	***		Sub-groups		***
	foreach subgroup of varlist $disaggregates{ //sample-weighted means for sub-groups
	
	levelsof `subgroup' , local(groups)
	foreach group in `groups'{
		
		* All
			svy, subpop(if `subgroup' == `group'): mean `outcome' if treatment == 0
			local ci = "-"
			local N = "-"
		if _rc==0{		
			matrix b = e(b)
			local mean : di %9.1f b[1,1] //sample-weighted mean, all households		
			matrix N = e(_N)
			local N : di %9.0f N[1,1] //N, all households
			
			svy, subpop(if `subgroup' == `group'): reg `outcome' treatment if inlist(treatment,0,1)
			starmaker treatment 3
			putexcel D`row' = "$stars"
				}
		putexcel B`row' = "`mean'" E`row' = "`mean'" 
		local row = `row' + 1
		}
	}
}
restore
end	

*---------------------------------------------*
*          SETUP: DATA AND VARIABLES          *			
*---------------------------------------------*

use "$data\food_cat_012.dta", clear
	mmerge hhid using "$data\cons_analysis.dta", unmatched (master) ukeep(cluster strata hhweight)
	rename treat treatment
	drop _merge
	recode fgr (1=1) (7=2) (2=3) (3=4) (4=5) (5=6) (6=7) (8=8) (10=9) (11=10) (9=11) (12=.), gen(fgr_ordered)

putexcel set "$tables\Tables and Figures - Consumption Experiment.xlsx", sheet(T5) modify
svyset [pw=hhweight], psu(cluster) strata(strata) singleunit(centered)
global indicators food_exp
global disaggregates "fgr_ordered"

ftftable "Tables and Figures - Consumption Experiment.xlsx" 2  

********************************************************************************
** Table 5: Average monthly expenditure of food groups, by treatment (T1)
********************************************************************************  

capture program drop ftftable // define the ftftable program
program define ftftable //ftftable was created by Greg Haugan at NORC 

preserve
putexcel set "$tables\Tables and Figures - Consumption Experiment.xlsx", sheet(T5) modify

local row = 3
foreach outcome of varlist $indicators {

	***		Sub-groups		***
	foreach subgroup of varlist $disaggregates{ //sample-weighted means for sub-groups
	
	levelsof `subgroup' , local(groups)
	foreach group in `groups'{
		
		* All
			svy, subpop(if `subgroup' == `group'): mean `outcome' if treatment == 1
			local ci = "-"
			local N = "-"
		if _rc==0{		
			matrix b = e(b)
			local mean : di %9.1f b[1,1] //sample-weighted mean, all households		
			matrix N = e(_N)
			local N : di %9.0f N[1,1] //N, all households
			
			svy, subpop(if `subgroup' == `group'): reg `outcome' treatment if inlist(treatment,0,2)
			starmaker treatment 3
			putexcel G`row' = "$stars"
				}
		putexcel C`row' = "`mean'" H`row' = "`mean'" 
		local row = `row' + 1
		}
	}
}
restore
end	

*---------------------------------------------*
*          SETUP: DATA AND VARIABLES          *			
*---------------------------------------------*

use "$data\food_cat_012.dta", clear
	mmerge hhid using "$data\cons_analysis.dta", unmatched (master) ukeep(cluster strata hhweight)
	rename treat treatment
	drop _merge
	recode fgr (1=1) (7=2) (2=3) (3=4) (4=5) (5=6) (6=7) (8=8) (10=9) (11=10) (9=11) (12=.), gen(fgr_ordered)

putexcel set "$tables\Tables and Figures - Consumption Experiment.xlsx", sheet(T5) modify
svyset [pw=hhweight], psu(cluster) strata(strata) singleunit(centered)
global indicators food_exp
global disaggregates "fgr_ordered"
ftftable "Tables and Figures - Consumption Experiment.xlsx" 2  

********************************************************************************
** Table 5: Average monthly expenditure of food groups, by treatment (T2)
********************************************************************************  

capture program drop ftftable // define the ftftable program
program define ftftable //ftftable was created by Greg Haugan at NORC 

preserve
putexcel set "$tables\Tables and Figures - Consumption Experiment.xlsx", sheet(T5) modify

local row = 3
foreach outcome of varlist $indicators {

	***		Sub-groups		***
	foreach subgroup of varlist $disaggregates{ //sample-weighted means for sub-groups
	
	levelsof `subgroup' , local(groups)
	foreach group in `groups'{
		
		* All
			svy, subpop(if `subgroup' == `group'): mean `outcome' if treatment == 2
			local ci = "-"
			local N = "-"
		if _rc==0{		
			matrix b = e(b)
			local mean : di %9.1f b[1,1] //sample-weighted mean, all households		
			matrix N = e(_N)
			local N : di %9.0f N[1,1] //N, all households
			
			svy, subpop(if `subgroup' == `group'): reg `outcome' treatment if inlist(treatment,1,2)
			starmaker treatment 3
			putexcel J`row' = "$stars"
				}
		putexcel F`row' = "`mean'" I`row' = "`mean'" 
		local row = `row' + 1
		}
	}
}
restore
end	

*---------------------------------------------*
*          SETUP: DATA AND VARIABLES          *			
*---------------------------------------------*

use "$data\food_cat_012.dta", clear
	mmerge hhid using "$data\cons_analysis.dta", unmatched (master) ukeep(cluster strata hhweight)
	rename treat treatment
	drop _merge
	recode fgr (1=1) (7=2) (2=3) (3=4) (4=5) (5=6) (6=7) (8=8) (10=9) (11=10) (9=11) (12=.), gen(fgr_ordered)

putexcel set "$tables\Tables and Figures - Consumption Experiment.xlsx", sheet(T5) modify
svyset [pw=hhweight], psu(cluster) strata(strata) singleunit(centered)
global indicators food_exp
global disaggregates "fgr_ordered"

ftftable "Tables and Figures - Consumption Experiment.xlsx" 2  

********************************************************************************
** Table 6. Average monthly food and non-food consumption expenditure, by treatment
*******************************************************************************
          
capture program drop ftftable 	// define the ftftable program
program define ftftable 		// ftftable was created by Greg Haugan at NORC 

preserve
local row = 3
foreach outcome of varlist $indicators{

	*** Results
svy: mean `outcome' if treatment == 0
matrix b = e(b)
local mean : di %9.0f b[1,1]
putexcel B`row' = "`mean'" E`row' = "`mean'" 

svy: mean `outcome' if treatment == 1
matrix b = e(b)
local mean : di %9.0f b[1,1]
putexcel C`row' = "`mean'" H`row' = "`mean'" 

svy: mean `outcome' if treatment == 2
matrix b = e(b)
local mean : di %9.0f b[1,1]
putexcel F`row' = "`mean'" I`row' = "`mean'" 

	***	Significance tests 	
svy: reg `outcome' treatment if inlist(treatment,0,1)
starmaker treatment 3
putexcel D`row' = "$stars"

svy: reg `outcome' treatment if inlist(treatment,0,2)
starmaker treatment 3
putexcel G`row' = "$stars"

svy: reg `outcome' treatment if inlist(treatment,1,2)
starmaker treatment 3
putexcel J`row' = "$stars"

local row = `row' + 1

}
restore
end   

*---------------------------------------------*
*          SETUP: DATA AND VARIABLES          *			
*---------------------------------------------*

use "$data\cons_analysis.dta", clear
// Table prep: generate total exp. on food var
	egen all_food=rowtotal(food_in food_out), missing
	egen all_nonfood=rowtotal(nonfood_7 nonfood_30 nonfood_365 edu_exp health_exp dur_exp rent_exp), missing
  
putexcel set "$tables\Tables and Figures - Consumption Experiment.xlsx", sheet(T6) modify

svyset [pw=hhweight], psu(cluster) strata(strata) singleunit(centered)
global indicators all_food xfood_in kfood_in food_out all_nonfood nonfood_7 nonfood_30 nonfood_365 edu_exp health_exp dur_exp rent_exp exp
ftftable "Tables and Figures - Consumption Experiment.xlsx" 2

********************************************************************************
** Table 7. Percentile of monthly food consumption expenditure, by treatment
********************************************************************************

// PERCENTILES            
use "$data\cons_analysis.dta", clear
// Table prep: generate total exp. on food variable
	egen all_food=rowtotal(food_in food_out), missing
   *egen all_foodexp= total(all_food), by(hhid) missing
  
tabstat all_food [aw=hhweight], stat(p10 p25 p50 p75 p90 mean)  by(treatment) columns(stat) f(%10.0f) save notot
tabstatmat A
	mat A=A

putexcel set "$tables\Tables and Figures - Consumption Experiment.xlsx", sheet(T7) modify
putexcel B3=mat(A), nformat(number_sep)	

********************************************************************************
** Table 7. Significance Test - Treatment 0 and 1
********************************************************************************

capture program drop ftftable 	// define the ftftable program
program define ftftable 		// ftftable was created by Greg Haugan at NORC 

preserve
local row = 4
foreach outcome of varlist $indicators{

	*** Treatment 0 and 1
ksmirnov  `outcome', by(treatment)
putexcel H`row' =`r(p)', nformat(number_d2)

}
restore
end   

*---------------------------------------------*
*          SETUP: DATA AND VARIABLES          *			
*---------------------------------------------*

use "$data\cons_analysis.dta", clear
keep if inlist(treatment,0,1)
// Table prep: generate total exp. on food variable
	egen all_food=rowtotal(food_in food_out), missing
 
putexcel set "$tables\Tables and Figures - Consumption Experiment.xlsx", sheet(T7) modify
global indicators all_food
ftftable "Tables and Figures - Consumption Experiment.xlsx" 2 

********************************************************************************
** Table 7. Significance Test - Treatment 0 and 2
********************************************************************************

capture program drop ftftable 	// define the ftftable program
program define ftftable 		// ftftable was created by Greg Haugan at NORC 

preserve
local row = 5
foreach outcome of varlist $indicators{

	*** Treatment 0 and 2
ksmirnov  `outcome', by(treatment)
putexcel H`row' =`r(p)', nformat(number_d2)

}
restore
end   

*---------------------------------------------*
*          SETUP: DATA AND VARIABLES          *			
*---------------------------------------------*

use "$data\cons_analysis.dta", clear
keep if inlist(treatment,0,2)
//Table prep: generate total exp. on food variable
	egen all_food=rowtotal(food_in food_out), missing
 
putexcel set "$tables\Tables and Figures - Consumption Experiment.xlsx", sheet(T7) modify
global indicators all_food
ftftable "Tables and Figures - Consumption Experiment.xlsx" 2 

*********************************************************************************************
** Table 8. Distribution of food group consumption and expenditure shares, by treatment
*********************************************************************************************
*************************
** Table 8. Treatment 0
*************************

capture program drop ftftable 	// define the ftftable program
program define ftftable 		// ftftable was created by Greg Haugan at NORC 

preserve
local row = 4
foreach outcome of varlist $indicators{

svy: mean `outcome' 
matrix b = e(b)
local mean : di %9.1f b[1,1]
putexcel B`row' = "`mean'" 

local row = `row' + 1

}
restore
end   

*---------------------------------------------*
*          SETUP: DATA AND VARIABLES          *			
*---------------------------------------------*

use "$data\food_cat_012.dta", clear
rename treat treatment

// Table prep
	keep if treatment==0
	tab fgr, gen(fgr_)
	collapse (max)fgr_*, by(hhid)
	recode fgr_* (1=100)
	
	mmerge hhid using "$data\cons_analysis.dta", unmatched (master) ukeep(cluster strata hhweight) 
	drop _merge fgr_num

putexcel set "$tables\Tables and Figures - Consumption Experiment.xlsx", sheet(T8) modify
svyset [pw=hhweight], psu(cluster) strata(strata) singleunit(centered)
global indicators fgr_*
ftftable "Tables and Figures - Consumption Experiment.xlsx" 2  
 
*************************
** Table 8. Treatment 1
*************************
	   
capture program drop ftftable 	// define the ftftable program
program define ftftable 		// ftftable was created by Greg Haugan at NORC 

preserve
local row = 4
foreach outcome of varlist $indicators{

svy: mean `outcome' 
matrix b = e(b)
local mean : di %9.1f b[1,1]
putexcel C`row' = "`mean'" 

local row = `row' + 1

}
restore
end   

*---------------------------------------------*
*          SETUP: DATA AND VARIABLES          *			
*---------------------------------------------*

use "$data\food_cat_012.dta", clear
rename treat treatment

// Table prep
	keep if treatment==1
	tab fgr, gen(fgr_)
	collapse (max)fgr_*, by(hhid)
	recode fgr_* (1=100)
	
	mmerge hhid using "$data\cons_analysis.dta", unmatched (master) ukeep(cluster strata hhweight) 
	drop _merge fgr_num

putexcel set "$tables\Tables and Figures - Consumption Experiment.xlsx", sheet(T8) modify
svyset [pw=hhweight], psu(cluster) strata(strata) singleunit(centered)
global indicators fgr_1-fgr_11
ftftable "Tables and Figures - Consumption Experiment.xlsx" 2 
 
*************************
** Table 8. Treatment 2
*************************
		 
capture program drop ftftable 	// define the ftftable program
program define ftftable 		// ftftable was created by Greg Haugan at NORC 

preserve
local row = 4
foreach outcome of varlist $indicators{

svy: mean `outcome' 
matrix b = e(b)
local mean : di %9.1f b[1,1]
putexcel D`row' = "`mean'" 

local row = `row' + 1

}
restore
end   

*---------------------------------------------*
*          SETUP: DATA AND VARIABLES          *			
*---------------------------------------------*

use "$data\food_cat_012.dta", clear
rename treat treatment

// Table prep
	keep if treatment==2
	tab fgr, gen(fgr_)
	collapse (max)fgr_*, by(hhid)
	recode fgr_* (1=100)

	mmerge hhid using "$data\cons_analysis.dta", unmatched (master) ukeep(cluster strata hhweight) 
	drop _merge fgr_num

putexcel set "$tables\Tables and Figures - Consumption Experiment.xlsx", sheet(T8) modify
svyset [pw=hhweight], psu(cluster) strata(strata) singleunit(centered)
global indicators fgr_*
ftftable "Tables and Figures - Consumption Experiment.xlsx" 2 

*********************************************************************************************
** Table 8. Distribution of food group consumption and expenditure shares - share in consumption
*********************************************************************************************
             	 
capture program drop ftftable 	// define the ftftable program
program define ftftable 		// ftftable was created by Greg Haugan at NORC 

preserve
local row = 4
foreach outcome of varlist $indicators{

svy: mean `outcome' if  treatment==0
matrix b = e(b)
local mean : di %9.1f b[1,1]*100
putexcel E`row' = "`mean'" 

svy: mean `outcome' if  treatment==1
matrix b = e(b)
local mean : di %9.1f b[1,1]*100
putexcel F`row' = "`mean'" 

svy: mean `outcome' if  treatment==2
matrix b = e(b)
local mean : di %9.1f b[1,1]*100
putexcel G`row' = "`mean'" 
local row = `row' + 1

}
restore
end   

*---------------------------------------------*
*          SETUP: DATA AND VARIABLES          *			
*---------------------------------------------*

use "$data\food_cat_012.dta", clear
rename treat treatment
drop fgr_num items_num
reshape wide food_exp, i(hhid) j(fgr)
egen tot_food_exp=rowtotal(food_exp*)

// Table prep
foreach v of varlist food_exp* {
	recode `v' (0=.)
	gen sh_`v' = `v'/tot_food_exp
	}

	mmerge hhid using "$data\cons_analysis.dta", unmatched (master) ukeep(cluster strata hhweight) 
	drop _merge  sh_food_exp12

putexcel set "$tables\Tables and Figures - Consumption Experiment.xlsx", sheet(T8) modify
svyset [pw=hhweight], psu(cluster) strata(strata) singleunit(centered)
global indicators sh_food_exp*
ftftable "Tables and Figures - Consumption Experiment.xlsx" 2 

********************************************************************************
** Table 9. Shares of monthly household food and non-food expenditure, by treatment
*******************************************************************************

capture program drop ftftable 	// define the ftftable program
program define ftftable 		// ftftable was created by Greg Haugan at NORC 

preserve
local row = 4
foreach outcome of varlist $indicators{

	*** Results
svy: mean `outcome' if treatment == 0
matrix b = e(b)
local mean : di %9.1f b[1,1]*100 
putexcel B`row' = "`mean'" 

svy: mean `outcome' if treatment == 1
matrix b = e(b)
local mean : di %9.1f b[1,1]*100 
putexcel C`row' = "`mean'" 

svy: mean `outcome' if treatment == 2
matrix b = e(b)
local mean : di %9.1f b[1,1]*100 
putexcel D`row' = "`mean'" 

local row = `row' + 1

}
restore
end   

*---------------------------------------------*
*          SETUP: DATA AND VARIABLES          *			
*---------------------------------------------*

use "$data\cons_analysis.dta", clear
keep if inlist(treatment,0,1,2)
// Table prep: generate total exp. on food var
	egen all_food=rowtotal(food_in food_out), missing
	egen all_nonfood=rowtotal(nonfood_7 nonfood_30 nonfood_365 edu_exp health_exp dur_exp rent_exp), missing
   
 foreach v of varlist all_food xfood_in kfood_in food_out all_nonfood nonfood_7 nonfood_30 nonfood_365 edu_exp health_exp dur_exp rent_exp {
	recode `v' (0=.)
	gen sh_`v' = `v'/exp
		}
	
putexcel set "$tables\Tables and Figures - Consumption Experiment.xlsx", sheet(T9) modify

svyset [pw=hhweight], psu(cluster) strata(strata) singleunit(centered)
global indicators sh_all_food sh_xfood_in sh_kfood_in sh_food_out sh_all_nonfood sh_nonfood_7 sh_nonfood_30 sh_nonfood_365 sh_edu_exp sh_health_exp sh_dur_exp sh_rent_exp 
ftftable "Tables and Figures - Consumption Experiment.xlsx" 2 

********************************************************************************
** TABLE 10: Prevalence of Poverty (%) - National Poverty Line (TSh 1,876)
********************************************************************************

capture program drop ftftable 	// define the ftftable program
program define ftftable 		// ftftable was created by Greg Haugan at NORC 

preserve
local row = 4
foreach outcome of varlist $indicators{
	
	**   Treatment 0 **
svy: mean `outcome' if treat==0
matrix b = e(b)
local mean : di %9.1f b[1,1]*100
local ll : di %9.1f _b[`outcome']*100 - (invttail(e(df_r),0.025) * _se[`outcome']*100)	//confidence interval					
local ul : di %9.1f _b[`outcome']*100 + (invttail(e(df_r),0.025) * _se[`outcome']*100)						
local ll = ltrim("`ll'")
local ul = ltrim("`ul'")
local ci = "[`ll' - `ul']"
putexcel B`row' = "`mean'" C`row' = ("`ci'") 
local row = `row' + 1

	**   Treatment 1 **
svy: mean `outcome' if treat==1
matrix b = e(b)
local mean : di %9.1f b[1,1]*100
local ll : di %9.1f _b[`outcome']*100 - (invttail(e(df_r),0.025) * _se[`outcome']*100)	//confidence interval					
local ul : di %9.1f _b[`outcome']*100 + (invttail(e(df_r),0.025) * _se[`outcome']*100)						
local ll = ltrim("`ll'")
local ul = ltrim("`ul'")
local ci = "[`ll' - `ul']"
putexcel B`row' = "`mean'" C`row' = ("`ci'") 
local row = `row' + 1

	**   Treatment 2 **
svy: mean `outcome' if treat==2
matrix b = e(b)
local mean : di %9.1f b[1,1]*100
local ll : di %9.1f _b[`outcome']*100 - (invttail(e(df_r),0.025) * _se[`outcome']*100)	//confidence interval					
local ul : di %9.1f _b[`outcome']*100 + (invttail(e(df_r),0.025) * _se[`outcome']*100)						
local ll = ltrim("`ll'")
local ul = ltrim("`ul'")
local ci = "[`ll' - `ul']"
putexcel B`row' = "`mean'" C`row' = ("`ci'") 
local row = `row' + 1

}
restore
end   
	
*---------------------------------------------*
*          SETUP: DATA AND VARIABLES          *			
*---------------------------------------------*
		
use "$data\cons_analysis.dta", clear
putexcel set "$tables\Tables and Figures - Consumption Experiment.xlsx", sheet(T10) modify

svyset [pw=hhweight2], psu(cluster) strata(strata) singleunit(centered)
global indicators poor1876
ftftable "Tables and Figures - Consumption Experiment.xlsx" 2 

********************************************************************************
** TABLE 10: Prevalence of Poverty (%) -  International Poverty Line (US$1.90 = TSh 1,644)
********************************************************************************

capture program drop ftftable 	// define the ftftable program
program define ftftable 		// ftftable was created by Greg Haugan at NORC 

preserve
local row = 4
foreach outcome of varlist $indicators{
	
	**   Treatment 0 **
svy: mean `outcome' if treat==0
matrix b = e(b)
local mean : di %9.1f b[1,1]*100
local ll : di %9.1f _b[`outcome']*100 - (invttail(e(df_r),0.025) * _se[`outcome']*100)	//confidence interval					
local ul : di %9.1f _b[`outcome']*100 + (invttail(e(df_r),0.025) * _se[`outcome']*100)						
local ll = ltrim("`ll'")
local ul = ltrim("`ul'")
local ci = "[`ll' - `ul']"
putexcel D`row' = "`mean'" E`row' = ("`ci'")
local row = `row' + 1

	**   Treatment 1 **
svy: mean `outcome' if treat==1
matrix b = e(b)
local mean : di %9.1f b[1,1]*100
local ll : di %9.1f _b[`outcome']*100 - (invttail(e(df_r),0.025) * _se[`outcome']*100)	//confidence interval					
local ul : di %9.1f _b[`outcome']*100 + (invttail(e(df_r),0.025) * _se[`outcome']*100)						
local ll = ltrim("`ll'")
local ul = ltrim("`ul'")
local ci = "[`ll' - `ul']"
putexcel D`row' = "`mean'" E`row' = ("`ci'")
local row = `row' + 1

	**   Treatment 2 **
svy: mean `outcome' if treat==2
matrix b = e(b)
local mean : di %9.1f b[1,1]*100
local ll : di %9.1f _b[`outcome']*100 - (invttail(e(df_r),0.025) * _se[`outcome']*100)	//confidence interval					
local ul : di %9.1f _b[`outcome']*100 + (invttail(e(df_r),0.025) * _se[`outcome']*100)						
local ll = ltrim("`ll'")
local ul = ltrim("`ul'")
local ci = "[`ll' - `ul']"
putexcel D`row' = "`mean'" E`row' = ("`ci'")
local row = `row' + 1

}
restore
end   
	
*---------------------------------------------*
*          SETUP: DATA AND VARIABLES          *			
*---------------------------------------------*
	
use "$data\cons_analysis.dta", clear
putexcel set "$tables\Tables and Figures - Consumption Experiment.xlsx", sheet(T10) modify

svyset [pw=hhweight2], psu(cluster) strata(strata) singleunit(centered)
global indicators poor190
ftftable "Tables and Figures - Consumption Experiment.xlsx" 2 

********************************************************************************
** TABLE 10: Prevalence of Poverty (%) -  International Poverty Line (US$2.15 = TSh 1,860)
********************************************************************************

capture program drop ftftable 	// define the ftftable program
program define ftftable 		// ftftable was created by Greg Haugan at NORC 

preserve
local row = 4
foreach outcome of varlist $indicators{
	
	**   Treatment 0 **
svy: mean `outcome' if treat==0
matrix b = e(b)
local mean : di %9.1f b[1,1]*100
local ll : di %9.1f _b[`outcome']*100 - (invttail(e(df_r),0.025) * _se[`outcome']*100)	//confidence interval					
local ul : di %9.1f _b[`outcome']*100 + (invttail(e(df_r),0.025) * _se[`outcome']*100)						
local ll = ltrim("`ll'")
local ul = ltrim("`ul'")
local ci = "[`ll' - `ul']"
putexcel F`row' = "`mean'" G`row' = ("`ci'")
local row = `row' + 1

	**   Treatment 1 **
svy: mean `outcome' if treat==1
matrix b = e(b)
local mean : di %9.1f b[1,1]*100
local ll : di %9.1f _b[`outcome']*100 - (invttail(e(df_r),0.025) * _se[`outcome']*100)	//confidence interval					
local ul : di %9.1f _b[`outcome']*100 + (invttail(e(df_r),0.025) * _se[`outcome']*100)						
local ll = ltrim("`ll'")
local ul = ltrim("`ul'")
local ci = "[`ll' - `ul']"
putexcel F`row' = "`mean'" G`row' = ("`ci'")
local row = `row' + 1

	**   Treatment 2 **
svy: mean `outcome' if treat==2
matrix b = e(b)
local mean : di %9.1f b[1,1]*100
local ll : di %9.1f _b[`outcome']*100 - (invttail(e(df_r),0.025) * _se[`outcome']*100)	//confidence interval					
local ul : di %9.1f _b[`outcome']*100 + (invttail(e(df_r),0.025) * _se[`outcome']*100)						
local ll = ltrim("`ll'")
local ul = ltrim("`ul'")
local ci = "[`ll' - `ul']"
putexcel F`row' = "`mean'" G`row' = ("`ci'")
local row = `row' + 1

}
restore
end   
	
*---------------------------------------------*
*          SETUP: DATA AND VARIABLES          *			
*---------------------------------------------*
	
use "$data\cons_analysis.dta", clear
putexcel set "$tables\Tables and Figures - Consumption Experiment.xlsx", sheet(T10) modify

svyset [pw=hhweight2], psu(cluster) strata(strata) singleunit(centered)
global indicators poor215
ftftable "Tables and Figures - Consumption Experiment.xlsx" 2 

********************************************************************************
** TABLE 10: Prevalence of Poverty (%) -  International Poverty Line (US$3.25 = TSh 2,812)
********************************************************************************

capture program drop ftftable 	// define the ftftable program
program define ftftable 		// ftftable was created by Greg Haugan at NORC 

preserve
local row = 4
foreach outcome of varlist $indicators{
	
	**   Treatment 0 **
svy: mean `outcome' if treat==0
matrix b = e(b)
local mean : di %9.1f b[1,1]*100
local ll : di %9.1f _b[`outcome']*100 - (invttail(e(df_r),0.025) * _se[`outcome']*100)	//confidence interval					
local ul : di %9.1f _b[`outcome']*100 + (invttail(e(df_r),0.025) * _se[`outcome']*100)						
local ll = ltrim("`ll'")
local ul = ltrim("`ul'")
local ci = "[`ll' - `ul']"
putexcel H`row' = "`mean'" I`row' = ("`ci'") J`row' = ("`N'") 
local row = `row' + 1

	**   Treatment 1 **
svy: mean `outcome' if treat==1
matrix b = e(b)
local mean : di %9.1f b[1,1]*100
local ll : di %9.1f _b[`outcome']*100 - (invttail(e(df_r),0.025) * _se[`outcome']*100)	//confidence interval					
local ul : di %9.1f _b[`outcome']*100 + (invttail(e(df_r),0.025) * _se[`outcome']*100)						
local ll = ltrim("`ll'")
local ul = ltrim("`ul'")
local ci = "[`ll' - `ul']"
putexcel H`row' = "`mean'" I`row' = ("`ci'") J`row' = ("`N'") 
local row = `row' + 1

	**   Treatment 2 **
svy: mean `outcome' if treat==2
matrix b = e(b)
local mean : di %9.1f b[1,1]*100
local ll : di %9.1f _b[`outcome']*100 - (invttail(e(df_r),0.025) * _se[`outcome']*100)	//confidence interval					
local ul : di %9.1f _b[`outcome']*100 + (invttail(e(df_r),0.025) * _se[`outcome']*100)						
local ll = ltrim("`ll'")
local ul = ltrim("`ul'")
local ci = "[`ll' - `ul']"
putexcel H`row' = "`mean'" I`row' = ("`ci'") J`row' = ("`N'") 
local row = `row' + 1

}
restore
end   
	
*---------------------------------------------*
*          SETUP: DATA AND VARIABLES          *			
*---------------------------------------------*
	
use "$data\cons_analysis.dta", clear
putexcel set "$tables\Tables and Figures - Consumption Experiment.xlsx", sheet(T10) modify

svyset [pw=hhweight2], psu(cluster) strata(strata) singleunit(centered)
global indicators poor325
ftftable "Tables and Figures - Consumption Experiment.xlsx" 2

********************************************************************************	
//	Table 10b: Prevalence of poverty, significance tests
********************************************************************************

capture program drop ftftable 	// define the ftftable program
program define ftftable 		// ftftable was created by Greg Haugan at NORC 

preserve
local row = 3
foreach outcome of varlist $indicators{

	*** Results
svy: mean `outcome' if treatment == 0
matrix b = e(b)
local mean : di %9.1f b[1,1]*100
putexcel B`row' = "`mean'" E`row' = "`mean'" 

svy: mean `outcome' if treatment == 1
matrix b = e(b)
local mean : di %9.1f b[1,1]*100
putexcel C`row' = "`mean'" H`row' = "`mean'" 

svy: mean `outcome' if treatment == 2
matrix b = e(b)
local mean : di %9.1f b[1,1]*100
putexcel F`row' = "`mean'" I`row' = "`mean'" 

	***	Significance tests 	
svy: reg `outcome' treatment if inlist(treatment,0,1)
starmaker treatment 3
putexcel D`row' = "$stars"

svy: reg `outcome' treatment if inlist(treatment,0,2)
starmaker treatment 3
putexcel G`row' = "$stars"

svy: reg `outcome' treatment if inlist(treatment,1,2)
starmaker treatment 3
putexcel J`row' = "$stars"

local row = `row' + 1

}
restore
end   

*---------------------------------------------*
*          SETUP: DATA AND VARIABLES          *			
*---------------------------------------------*

use "$data\cons_analysis.dta", clear
putexcel set "$tables\Tables and Figures - Consumption Experiment.xlsx", sheet(T10B) modify

svyset [pw=hhweight2], psu(cluster) strata(strata) singleunit(centered)
global indicators poor1876 poor190 poor215 poor325
ftftable "Tables and Figures - Consumption Experiment.xlsx" 2

********************************************************************************	
//	Figure 1: Daily Food Consumption Expenditure - by day (14 days)
********************************************************************************
	
** Set-up data
	use "$data\cons_analysis_day.dta", clear
	mmerge hhid using "$data\cons_analysis.dta", ukeep(hhweight cluster strata)
	keep if _merge==3
	sort hhid day
	svyset [pw=hhweight], psu(cluster) strata(strata) singleunit(centered)

		** FIGURE:
		** Daily Food Expenditure 
			clear matrix
			foreach var of varlist food_total_day food_home_day food_out_day {
				forvalues i = 1/14 {
				svy: mean `var' if day == `i'
				matrix Q1 = e(b)
					local mean : di %9.1f Q1[1,1]
					mat `var'`i' = nullmat(`var'`i')\(Q1)
						mat Main=nullmat(Main)\(`var'`i')					
							}
						}
				mat def A = Main[1..14,1...]
				mat def B = Main[15..28,1...]
				mat def C = Main[29..42,1...]	
		
		** Number of Observations
			foreach var of varlist day {
				forvalues i = 1/14 {
				svy: mean `var' if day == `i'
				matrix Q2 = e(N)
					local mean : di %9.1f Q2[1,1]
					mat `var'`i' = nullmat(`var'`i')\(Q2)
						mat Days=nullmat(Days)\(`var'`i')			
					}
				}
			mat D = A,B,C,Days
			mat Figure1 = D' 
			putexcel set "$tables\Tables and Figures - Consumption Experiment.xlsx", sheet(F1) mod
				putexcel C3=matrix(Figure1[1...,1...])
 	
********************************************************************************		
//	Figure 2. CDF of total household expenditure, by treatment
********************************************************************************	

	use "$data\cons_analysis.dta", clear
	recode treatment (0=1 "T0") (1=2 "T1") (2=3 "T2"), gen(treat)

		loc var exp_aeq 
		loc lim 60000
		local x : var label `var'
		cdf `var', hsize(hhweight2) hgroup(treat) min(50000) max(60000) ///
			scheme(s1color) c(J J J) lc(blue red green)  ///
			xline(52047.14 54851.02 57344.25, lc(pink*0.5) lp(dash)) ///
			xtitle("`x'") legend(ring(1) pos(12) rows(1) size(small) ) ///
			xla(50000(1000)60000, format ("%04.0fc") ang(h) labsize(small)) ///
			yla(0.1(0.02)0.30, format ("%03.2f") ang(h) labsize(small)) ///
			yline(.1907 .1988  .1888, lc(grey*0.3) lp(dash))
		 graph save "$tables\Figure 2.gph", replace
	graph export "$tables\Figure 2.png", replace
		 
			
********************************************************************************	
//	Figure 3. Lorenz curves of inequality, by treatment
********************************************************************************

		use "$data\cons_analysis.dta", clear
		label define treatment 0 "T0" 1 "T1" 2 "T2"
		label val treat treatment
		 
			lorenz estimate exp_aeq [w=hhweight*hhsize], over(treat) graph(aspectratio(1))
			lorenz graph, overlay aspectratio(0.8) xlabel(0(10)100, grid) graphregion(color(white)) ///
			bgcolor(white) xtitle("") ytitle("") legend(ring(1) pos(12) rows(1))
		graph save "$tables\Figure 3.gph", replace

	graph export "$tables\Figure 3.png", replace

********************************************************************************
********************************************************************************