
	//	Call the Analytical Datafile
		use "$data\cons_analysis.dta", clear

     //	Generate and Label Relevant Variables
		
		****label treatment variable
		lab define treatment  0 "14-Day Diary" 1 "7-Day short" 2 "7-Day long"
		lab values treatment treatment

		***employment of HH head and main respondent
		recode wage_head (100=1) 
		recode nfe_head (100=1) (2=0)

		recode wage_resp (100=1)
		recode nfe_resp  (100=1)

		***sex of main respondent
		recode sex_resp  (100=1)	
		
		** Generate floor materials variable
		tab dw_floor, gen(dw_floor_)

		*** generate drinking water variables
		recode safewater_dry (100=1)
		recode safewater_rainy2 (100=1)
		 
		   ***generate total food exp. var
		egen all_food=rowtotal(food_in food_out), missing
		gen ln_all_food=ln(all_food/hhsize)

		   ***generate total consumption exp. var
		gen ln_exp=ln(exp/hhsize)

	//	Set the Data as a Survey Data (Svy set data)
		svyset [pw=hhweight], psu(cluster) strata(strata) singleunit(centered) 

 
	********************************************************************************
	** TABLE 11a: Regression of food expenditure on type of consumption module - HH head
	********************************************************************************
	*** Model 1 - No Controls
		svy: reg ln_all_food i.treatment 
			outreg2 using "$tables\Regression_tables_11a.xls", paren(se) bdec(3) replace ctitle(Model 1) ///
				label title(TABLE 1a: Regression of food expenditure on type of consumption module - Household head)

	*** Model 2 - With Controls but No interaction	
		svy: reg ln_all_food i.treatment seceduc_head work_head nfe_head dw_floor_2 safewater_rainy2 safewater_dry
			outreg2 using "$tables\Regression_tables_11a.xls", paren(se) bdec(3) keep(1.treatment 2.treatment) ///
				append ctitle(Model 2) label

	***	Model 3 - Interaction with education  	
		svy: reg ln_all_food i.treatment##seceduc_head 
			outreg2 using "$tables\Regression_tables_11a.xls", paren(se) bdec(3) ///
				keep(1.treatment 2.treatment 1.treatment#1.seceduc_head 2.treatment#1.seceduc_head) append ctitle(Model 3) label

	***	Model 4 - Interaction with employment status	
		svy: reg ln_all_food i.treatment##work_head
			outreg2 using "$tables\Regression_tables_11a.xls", paren(se) bdec(3) ///
				keep(1.treatment 2.treatment 1.treatment#1.work_head 2.treatment#1.work_head) append ctitle(Model 4) label

	***Model 5 - Interaction with employment type (NFE)
		svy: reg ln_all_food i.treatment##nfe_head
			outreg2 using "$tables\Regression_tables_11a.xls", paren(se) bdec(3) ///
				keep(1.treatment 2.treatment 1.treatment#1.nfe_head 2.treatment#1.nfe_head) append ctitle(Model 5) label

	***	Model 6 - Interaction with floor materials
		svy: reg ln_all_food i.treatment##dw_floor_2
			outreg2 using "$tables\Regression_tables_11a.xls", paren(se) bdec(3) ///
				keep(1.treatment 2.treatment 1.treatment#1.dw_floor_2 2.treatment#1.dw_floor_2) append ctitle(Model 6) label

	***	Model 7 - Interaction with drinking water, rainy season
		svy: reg ln_all_food i.treatment##safewater_rainy2
			outreg2 using "$tables\Regression_tables_11a.xls", paren(se) bdec(3) ///
				keep(1.treatment 2.treatment 1.treatment#1.safewater_rainy2 2.treatment#1.safewater_rainy2) append ctitle(Model 7) label

	***	Model 8 - Interaction with drinking water, dry season
		svy: reg ln_all_food i.treatment##safewater_dry 
			outreg2 using "$tables\Regression_tables_11a.xls", paren(se) bdec(3) ///
				keep(1.treatment 2.treatment 1.treatment#1.safewater_dry 2.treatment#1.safewater_dry) append ctitle(Model 8) label

	***	Model 9 - Interaction with education, employment status & type, floor materials, drinking water
		svy: reg ln_all_food i.treatment##seceduc_head i.treatment##work_head i.treatment##nfe_head i.treatment##dw_floor_2 ///
				 i.treatment##safewater_rainy2 i.treatment##safewater_dry 
			outreg2 using "$tables\Regression_tables_11a.xls", paren(se) bdec(3) ///
				keep(1.treatment 2.treatment 1.treatment#1.seceduc_head 2.treatment#1.seceduc_head 1.treatment#1.work_head ///
				2.treatment#1.work_head 1.treatment#1.nfe_head 2.treatment#1.nfe_head 1.treatment#1.dw_floor_2 ///
				2.treatment#1.dw_floor_2 1.treatment#1.safewater_rainy2 2.treatment#1.safewater_rainy2 ///
				1.treatment#1.safewater_dry 2.treatment#1.safewater_dry) append ctitle(Model 9) label  
				

	********************************************************************************
	** TABLE 11b: Regression of food expenditure on treatment type - Main respondent
	********************************************************************************
	*** Model 1 - No interaction	
		svy: reg ln_all_food i.treatment
			outreg2 using "$tables\Regression_tables_11b.xls", paren(se) bdec(3) keep(1.treatment 2.treatment) replace ///
				ctitle(Model 1) label title(TABLE 1b: Regression of food expenditure on type of consumption module - Main respondent)

	*** Model 2 - With Controls but No interaction   	
		svy: reg ln_all_food i.treatment sex_resp work_resp nfe_resp dw_floor_2 safewater_rainy2 safewater_dry
			outreg2 using "$tables\Regression_tables_11b.xls", paren(se) bdec(3) keep(1.treatment 2.treatment) ///
				append ctitle(Model 2) label

	***	Model 3 - Interaction with gender 	
		svy: reg ln_all_food i.treatment##sex_resp 
			outreg2 using "$tables\Regression_tables_11b.xls", paren(se) bdec(3) ///
				keep(1.treatment 2.treatment 1.treatment#1.sex_resp 2.treatment#1.sex_resp) append ctitle(Model 3) label

	***	Model 4 - Interaction with employment status
		svy: reg ln_all_food i.treatment##work_resp
			outreg2 using "$tables\Regression_tables_11b.xls", paren(se) bdec(3) ///
				keep(1.treatment 2.treatment 1.treatment#1.work_resp 2.treatment#1.work_resp) append ctitle(Model 4) label

	***	Model 5 - Interaction with employment type (NFE)
		svy: reg ln_all_food i.treatment##nfe_resp
			outreg2 using "$tables\Regression_tables_11b.xls", paren(se) bdec(3) ///
				keep(1.treatment 2.treatment 1.treatment#1.nfe_resp 2.treatment#1.nfe_resp) append ctitle(Model 5) label

	***	Model 6 - Interaction with floor materials
		svy: reg ln_all_food i.treatment##dw_floor_2
			outreg2 using "$tables\Regression_tables_11b.xls", paren(se) bdec(3) ///
				keep(1.treatment 2.treatment 1.treatment#1.dw_floor_2 2.treatment#1.dw_floor_2) append ctitle(Model 6) label

	***	Model 7 - Interaction with drinking water, rainy season
		svy: reg ln_all_food i.treatment##safewater_rainy2
			outreg2 using "$tables\Regression_tables_11b.xls", paren(se) bdec(3) ///
				keep(1.treatment 2.treatment 1.treatment#1.safewater_rainy2 2.treatment#1.safewater_rainy2) append ctitle(Model 7) label

	***Model 8 - Interaction with drinking water, dry season
		svy: reg ln_all_food i.treatment##safewater_dry 
			outreg2 using "$tables\Regression_tables_11b.xls", paren(se) bdec(3) ///
				keep(1.treatment 2.treatment 1.treatment#1.safewater_dry 2.treatment#1.safewater_dry) append ctitle(Model 8) label

	***	Model 9 - Interaction with age, gender, education,and employment
		svy: reg ln_all_food i.treatment##sex_resp i.treatment##work_resp i.treatment##nfe_resp i.treatment##dw_floor_2 ///
				 i.treatment##safewater_rainy2 i.treatment##safewater_dry
			outreg2 using "$tables\Regression_tables_11b.xls", paren(se) bdec(3) ///
				keep(1.treatment 2.treatment 1.treatment#1.sex_resp 2.treatment#1.sex_resp 1.treatment#1.work_resp ///
				2.treatment#1.work_resp 1.treatment#1.nfe_resp 2.treatment#1.nfe_resp 1.treatment#1.dw_floor_2 ///
				2.treatment#1.dw_floor_2 1.treatment#1.safewater_rainy2 2.treatment#1.safewater_rainy2 ///
				1.treatment#1.safewater_dry 2.treatment#1.safewater_dry) append ctitle(Model 9) label 
				
				
				
	********************************************************************************
	** TABLE 12a: Regression of Total Consumption Expenditure on treatment Type - Household Head
	********************************************************************************
	*** Model 1 - No Controls
		svy: reg ln_exp i.treatment 
			outreg2 using "$tables\Regression_tables_12a.xls", paren(se) bdec(3) replace ctitle(Model 1) ///
				label title(TABLE 1a: Regression of food expenditure on type of consumption module - Household head)

	*** Model 2 - With Controls but No interaction	
		svy: reg ln_exp i.treatment seceduc_head work_head nfe_head dw_floor_2 safewater_rainy2 safewater_dry
			outreg2 using "$tables\Regression_tables_12a.xls", paren(se) bdec(3) keep(1.treatment 2.treatment) ///
				append ctitle(Model 2) label

	***	Model 3 - Interaction with education  	
		svy: reg ln_exp i.treatment##seceduc_head 
			outreg2 using "$tables\Regression_tables_12a.xls", paren(se) bdec(3) ///
				keep(1.treatment 2.treatment 1.treatment#1.seceduc_head 2.treatment#1.seceduc_head) append ctitle(Model 3) label

	***	Model 4 - Interaction with employment status	
		svy: reg ln_exp i.treatment##work_head
			outreg2 using "$tables\Regression_tables_12a.xls", paren(se) bdec(3) ///
				keep(1.treatment 2.treatment 1.treatment#1.work_head 2.treatment#1.work_head) append ctitle(Model 4) label

	***Model 5 - Interaction with employment type (NFE)
		svy: reg ln_exp i.treatment##nfe_head
			outreg2 using "$tables\Regression_tables_12a.xls", paren(se) bdec(3) ///
				keep(1.treatment 2.treatment 1.treatment#1.nfe_head 2.treatment#1.nfe_head) append ctitle(Model 5) label

	***	Model 6 - Interaction with floor materials
		svy: reg ln_exp i.treatment##dw_floor_2
			outreg2 using "$tables\Regression_tables_12a.xls", paren(se) bdec(3) ///
				keep(1.treatment 2.treatment 1.treatment#1.dw_floor_2 2.treatment#1.dw_floor_2) append ctitle(Model 6) label

	***	Model 7 - Interaction with drinking water, rainy season
		svy: reg ln_exp i.treatment##safewater_rainy2
			outreg2 using "$tables\Regression_tables_12a.xls", paren(se) bdec(3) ///
				keep(1.treatment 2.treatment 1.treatment#1.safewater_rainy2 2.treatment#1.safewater_rainy2) append ctitle(Model 7) label

	***	Model 8 - Interaction with drinking water, dry season
		svy: reg ln_exp i.treatment##safewater_dry 
			outreg2 using "$tables\Regression_tables_12a.xls", paren(se) bdec(3) ///
				keep(1.treatment 2.treatment 1.treatment#1.safewater_dry 2.treatment#1.safewater_dry) append ctitle(Model 8) label

	***	Model 9 - Interaction with education, employment status & type, floor materials, drinking water
		svy: reg ln_exp i.treatment##seceduc_head i.treatment##work_head i.treatment##nfe_head i.treatment##dw_floor_2 ///
				 i.treatment##safewater_rainy2 i.treatment##safewater_dry 
			outreg2 using "$tables\Regression_tables_12a.xls", paren(se) bdec(3) ///
				keep(1.treatment 2.treatment 1.treatment#1.seceduc_head 2.treatment#1.seceduc_head 1.treatment#1.work_head ///
				2.treatment#1.work_head 1.treatment#1.nfe_head 2.treatment#1.nfe_head 1.treatment#1.dw_floor_2 ///
				2.treatment#1.dw_floor_2 1.treatment#1.safewater_rainy2 2.treatment#1.safewater_rainy2 ///
				1.treatment#1.safewater_dry 2.treatment#1.safewater_dry) append ctitle(Model 9) label  

				
				
	********************************************************************************
	** TABLE 12b: Regression of Total Consumption Expenditure on treatment Type - Main respondent
	********************************************************************************
	*** Model 1 - No interaction	
		svy: reg ln_exp i.treatment
		outreg2 using "$tables\Regression_tables_12b.xls", paren(se) bdec(3) keep(1.treatment 2.treatment) replace ///
			ctitle(Model 1) label title(TABLE 1b: Regression of food expenditure on type of consumption module - Main respondent)

	*** Model 2 - With Controls but No interaction   	
		svy: reg ln_exp i.treatment sex_resp work_resp nfe_resp dw_floor_2 safewater_rainy2 safewater_dry
		outreg2 using "$tables\Regression_tables_12b.xls", paren(se) bdec(3) keep(1.treatment 2.treatment) ///
			append ctitle(Model 2) label

	***	Model 3 - Interaction with gender 	
		svy: reg ln_exp i.treatment##sex_resp 
		outreg2 using "$tables\Regression_tables_12b.xls", paren(se) bdec(3) ///
			keep(1.treatment 2.treatment 1.treatment#1.sex_resp 2.treatment#1.sex_resp) append ctitle(Model 3) label

	***	Model 4 - Interaction with employment status
		svy: reg ln_exp i.treatment##work_resp
			outreg2 using "$tables\Regression_tables_12b.xls", paren(se) bdec(3) ///
				keep(1.treatment 2.treatment 1.treatment#1.work_resp 2.treatment#1.work_resp) append ctitle(Model 4) label

	***	Model 5 - Interaction with employment type (NFE)
		svy: reg ln_exp i.treatment##nfe_resp
			outreg2 using "$tables\Regression_tables_12b.xls", paren(se) bdec(3) ///
				keep(1.treatment 2.treatment 1.treatment#1.nfe_resp 2.treatment#1.nfe_resp) append ctitle(Model 5) label

	***	Model 6 - Interaction with floor materials
		svy: reg ln_exp i.treatment##dw_floor_2
			outreg2 using "$tables\Regression_tables_12b.xls", paren(se) bdec(3) ///
			keep(1.treatment 2.treatment 1.treatment#1.dw_floor_2 2.treatment#1.dw_floor_2) append ctitle(Model 6) label

	***	Model 7 - Interaction with drinking water, rainy season
		svy: reg ln_exp i.treatment##safewater_rainy2
			outreg2 using "$tables\Regression_tables_12b.xls", paren(se) bdec(3) ///
				keep(1.treatment 2.treatment 1.treatment#1.safewater_rainy2 2.treatment#1.safewater_rainy2) append ctitle(Model 7) label

	***Model 8 - Interaction with drinking water, dry season
		svy: reg ln_exp i.treatment##safewater_dry 
			outreg2 using "$tables\Regression_tables_12b.xls", paren(se) bdec(3) ///
				keep(1.treatment 2.treatment 1.treatment#1.safewater_dry 2.treatment#1.safewater_dry) append ctitle(Model 8) label

	***	Interaction with age, gender, education,and employment
		svy: reg ln_exp i.treatment##sex_resp i.treatment##work_resp i.treatment##nfe_resp i.treatment##dw_floor_2 ///
				 i.treatment##safewater_rainy2 i.treatment##safewater_dry
			outreg2 using "$tables\Regression_tables_12b.xls", paren(se) bdec(3) ///
				keep(1.treatment 2.treatment 1.treatment#1.sex_resp 2.treatment#1.sex_resp 1.treatment#1.work_resp ///
				2.treatment#1.work_resp 1.treatment#1.nfe_resp 2.treatment#1.nfe_resp 1.treatment#1.dw_floor_2 ///
				2.treatment#1.dw_floor_2 1.treatment#1.safewater_rainy2 2.treatment#1.safewater_rainy2 ///
				1.treatment#1.safewater_dry 2.treatment#1.safewater_dry) append ctitle(Model 9) label 
